api_testka_keys: list = [
    "AT_test_api_method", "AT_delegate_async_httpx", "AT_test_api_method_httpx",
    "AT_generate_html", "AT_generate_html_report", "AT_generate_json",
    "AT_generate_json_report", "AT_generate_xml", "AT_generate_xml_report", "AT_execute_action",
    "AT_execute_files", "AT_add_package_to_executor", "AT_add_package_to_callback_executor",
    "AT_flask_mock_server_add_router", "AT_start_flask_mock_server", "AT_scheduler_event_trigger",
    "AT_remove_blocking_scheduler_job", "AT_remove_nonblocking_scheduler_job",
    "AT_start_blocking_scheduler", "AT_start_nonblocking_scheduler",
    "AT_start_all_scheduler", "AT_shutdown_blocking_scheduler", "AT_shutdown_nonblocking_scheduler",
    "test_api_method", "add_command_to_executor", "execute_action", "execute_files", "executor",
    "get_dir_files_as_list", "generate_html", "generate_html_report", "read_action_json",
    "write_action_json", "reformat_json", "generate_json", "generate_json_report",
    "start_apitestka_socket_server",
    "test_record_instance", "dict_to_elements_tree", "elements_tree_to_dict",
    "XMLParser", "reformat_xml_file", "generate_xml", "generate_xml_report",
    "callback_executor", "create_project_dir", "flask_mock_server_instance",
    "request", "redirect", "SchedulerManager"
]

auto_control_keys: list = [
    "AC_mouse_left", "AC_mouse_right", "AC_mouse_middle", "AC_click_mouse", "AC_get_mouse_table",
    "AC_get_mouse_position", "AC_press_mouse", "AC_release_mouse", "AC_mouse_scroll",
    "AC_set_mouse_position", "AC_get_special_table", "AC_get_keyboard_keys_table",
    "AC_type_keyboard", "AC_press_keyboard_key", "AC_release_keyboard_key", "AC_check_key_is_press",
    "AC_write", "AC_hotkey", "AC_locate_all_image", "AC_locate_image_center",
    "AC_locate_and_click", "AC_screen_size", "AC_screenshot", "AC_set_record_enable", "AC_generate_html",
    "AC_generate_json", "AC_generate_xml", "AC_generate_html_report", "AC_generate_json_report",
    "AC_generate_xml_report", "AC_record", "AC_stop_record", "AC_execute_action",
    "AC_execute_files", "AC_add_package_to_executor", "AC_add_package_to_callback_executor",
    "AC_create_project", "AC_shell_command", "AC_execute_process", "AC_scheduler_event_trigger",
    "AC_remove_blocking_scheduler_job", "AC_remove_nonblocking_scheduler_job",
    "AC_start_blocking_scheduler", "AC_start_nonblocking_scheduler", "AC_start_all_scheduler",
    "AC_shutdown_blocking_scheduler", "AC_shutdown_nonblocking_scheduler",
    "click_mouse", "mouse_keys_table", "get_mouse_position", "press_mouse", "release_mouse",
    "mouse_scroll", "set_mouse_position", "special_mouse_keys_table",
    "keyboard_keys_table", "press_keyboard_key", "release_keyboard_key", "type_keyboard", "check_key_is_press",
    "write", "hotkey", "start_exe", "SchedulerManager", "get_keyboard_keys_table",
    "screen_size", "screenshot", "locate_all_image", "locate_image_center", "locate_and_click",
    "CriticalExit", "AutoControlException", "AutoControlKeyboardException",
    "AutoControlMouseException", "AutoControlCantFindKeyException",
    "AutoControlScreenException", "ImageNotFoundException", "AutoControlJsonActionException",
    "AutoControlRecordException", "AutoControlActionNullException", "AutoControlActionException", "record",
    "stop_record", "read_action_json", "write_action_json", "execute_action", "execute_files", "executor",
    "add_command_to_executor", "multiprocess_timeout", "test_record_instance", "screenshot", "pil_screenshot",
    "generate_html", "generate_html_report", "generate_json", "generate_json_report", "generate_xml",
    "generate_xml_report", "get_dir_files_as_list", "create_project_dir", "start_autocontrol_socket_server",
    "callback_executor", "package_manager", "get_special_table", "ShellManager", "default_shell_manager",
]

web_runner_keys: list = [
    "WR_get_webdriver_manager", "WR_change_index_of_webdriver", "WR_quit", "WR_SaveTestObject",
    "WR_CleanTestObject", "WR_set_driver", "WR_set_webdriver_options_capability", "WR_find_element",
    "WR_find_elements", "WR_implicitly_wait", "WR_explict_wait", "WR_to_url",
    "WR_forward", "WR_back", "WR_refresh", "WR_switch", "WR_set_script_timeout", "WR_set_page_load_timeout",
    "WR_get_cookies", "WR_get_cookie", "WR_add_cookie", "WR_delete_cookie", "WR_delete_all_cookies",
    "WR_execute", "WR_execute_script", "WR_execute_async_script", "WR_move_to_element",
    "WR_move_to_element_with_offset", "WR_drag_and_drop", "WR_drag_and_drop_offset", "WR_perform",
    "WR_reset_actions", "WR_left_click", "WR_left_click_and_hold", "WR_right_click",
    "WR_left_double_click", "WR_release", "WR_press_key", "WR_release_key",
    "WR_move_by_offset", "WR_pause", "WR_send_keys", "WR_send_keys_to_element", "WR_scroll",
    "WR_check_current_webdriver", "WR_maximize_window", "WR_fullscreen_window", "WR_minimize_window",
    "WR_set_window_size", "WR_set_window_position", "WR_get_window_position",
    "WR_get_window_rect", "WR_set_window_rect", "WR_get_screenshot_as_png", "WR_get_screenshot_as_base64",
    "WR_get_log", "WR_single_quit", "WR_element_submit", "WR_element_clear", "WR_element_get_property",
    "WR_element_get_dom_attribute", "WR_element_get_attribute",
    "WR_element_is_selected", "WR_element_is_enabled", "WR_input_to_element", "WR_click_element",
    "WR_element_is_displayed", "WR_element_value_of_css_property", "WR_element_screenshot",
    "WR_element_change_web_element", "WR_element_check_current_web_element", "WR_element_get_select",
    "WR_set_record_enable", "WR_generate_html", "WR_generate_html_report", "WR_generate_json",
    "WR_generate_json_report", "WR_generate_xml", "WR_generate_xml_report",
    "WR_execute_action", "WR_execute_files", "WR_add_package_to_executor",
    "WR_add_package_to_callback_executor", "WR_scheduler_event_trigger", "WR_remove_blocking_scheduler_job",
    "WR_remove_nonblocking_scheduler_job", "WR_start_blocking_scheduler", "WR_start_nonblocking_scheduler",
    "WR_start_all_scheduler", "WR_shutdown_blocking_scheduler", "WR_shutdown_nonblocking_scheduler",
    "web_element_wrapper", "set_webdriver_options_argument", "SchedulerManager",
    "webdriver_wrapper", "get_webdriver_manager",
    "get_desired_capabilities", "get_desired_capabilities_keys", "add_command_to_executor",
    "execute_action", "execute_files", "executor",
    "generate_html", "generate_html_report",
    "generate_json", "generate_json_report", "read_action_json",
    "generate_xml", "generate_xml_report",
    "start_web_runner_socket_server", "get_dir_files_as_list",
    "TestObject", "create_test_object", "get_test_object_type_list",
    "test_record_instance", "Keys", "callback_executor", "create_project_dir"
]

load_density_keys: list = [
    "LD_start_test", "LD_generate_html", "LD_generate_html_report", "LD_generate_json", "LD_generate_json_report",
    "LD_generate_xml", "LD_generate_xml_report", "LD_execute_action", "LD_execute_files",
    "LD_add_package_to_executor", "LD_scheduler_event_trigger", "LD_remove_blocking_scheduler_job",
    "LD_remove_nonblocking_scheduler_job", "LD_start_blocking_scheduler", "LD_start_nonblocking_scheduler",
    "LD_start_all_scheduler", "LD_shutdown_blocking_scheduler", "LD_shutdown_nonblocking_scheduler",
    "create_env", "LD_start_test", "SchedulerManager",
    "locust_wrapper_proxy",
    "prepare_env", "prepare_env",
    "test_record_instance",
    "execute_action", "execute_files", "executor", "add_command_to_executor",
    "get_dir_files_as_list",
    "generate_html", "generate_html_report",
    "generate_json", "generate_json_report", "read_action_json",
    "generate_xml", "generate_xml_report",
    "start_load_density_socket_server",
    "SequentialTaskSet", "task", "TaskSet",
    "callback_executor", "create_project_dir"
]

automation_file_keys: list = [
    "FA_create_file", "FA_copy_file", "FA_rename_file", "FA_remove_file", "FA_copy_all_file_to_dir",
    "FA_copy_specify_extension_file", "FA_copy_dir", "FA_create_dir", "FA_remove_dir_tree",
    "FA_zip_dir", "FA_zip_file", "FA_zip_info", "FA_zip_file_info", "FA_set_zip_password", "FA_unzip_file",
    "FA_read_zip_file", "FA_unzip_all", "FA_drive_later_init",
    "FA_drive_search_all_file", "FA_drive_search_field", "FA_drive_search_file_mimetype",
    "FA_drive_upload_dir_to_folder", "FA_drive_upload_to_folder", "FA_drive_upload_dir_to_drive",
    "FA_drive_upload_to_drive", "FA_drive_add_folder", "FA_drive_share_file_to_anyone",
    "FA_drive_share_file_to_domain", "FA_drive_share_file_to_user", "FA_drive_delete_file",
    "FA_drive_download_file", "FA_drive_download_file_from_folder",
    "FA_execute_action", "FA_execute_files", "FA_add_package_to_executor",
    "FA_scheduler_event_trigger", "FA_remove_blocking_scheduler_job", "FA_remove_nonblocking_scheduler_job",
    "FA_start_blocking_scheduler", "FA_start_nonblocking_scheduler", "FA_start_all_scheduler",
    "FA_shutdown_blocking_scheduler", "FA_shutdown_nonblocking_scheduler",
    "copy_file", "rename_file", "remove_file", "copy_all_file_to_dir", "copy_specify_extension_file",
    "copy_dir", "create_dir", "remove_dir_tree", "zip_dir", "zip_file", "zip_info",
    "zip_file_info", "set_zip_password", "unzip_file", "read_zip_file",
    "unzip_all", "driver_instance", "drive_search_all_file", "drive_search_field", "drive_search_file_mimetype",
    "drive_upload_dir_to_folder", "drive_upload_to_folder", "drive_upload_dir_to_drive", "drive_upload_to_drive",
    "drive_add_folder", "drive_share_file_to_anyone", "drive_share_file_to_domain", "drive_share_file_to_user",
    "drive_delete_file", "drive_download_file", "drive_download_file_from_folder", "execute_action", "execute_files",
    "add_command_to_executor", "read_action_json", "get_dir_files_as_list", "create_project_dir"
]

mail_thunder_keys: list = [
    "MT_smtp_later_init", "MT_smtp_create_message_with_attach_and_send", "MT_smtp_create_message_and_send",
    "MT_smtp_quit", "MT_imap_later_init", "MT_imap_select_mailbox", "MT_imap_search_mailbox",
    "MT_imap_mail_content_list", "MT_imap_output_all_mail_as_file", "MT_imap_quit",
    "MT_set_mail_thunder_os_environ", "MT_get_mail_thunder_os_environ", "MT_add_package_to_executor",
    "MT_scheduler_event_trigger", "MT_remove_blocking_scheduler_job", "MT_remove_nonblocking_scheduler_job",
    "MT_start_blocking_scheduler", "MT_start_nonblocking_scheduler", "MT_start_all_scheduler",
    "MT_shutdown_blocking_scheduler", "MT_shutdown_nonblocking_scheduler",
    "IMAPWrapper", "imap_instance", "SMTPWrapper", "smtp_instance", "is_need_to_save_content",
    "mail_thunder_content_data_dict", "read_output_content", "write_output_content", "get_mail_thunder_os_environ",
    "set_mail_thunder_os_environ", "read_action_json", "get_dir_files_as_list", "execute_action", "execute_files",
    "add_command_to_executor", "create_project_dir"
]

package_keyword_list = {
    "je_auto_control": auto_control_keys,
    "je_load_density": load_density_keys,
    "je_api_testka": api_testka_keys,
    "je_web_runner": web_runner_keys,
    "automation_file": automation_file_keys,
    "mail_thunder": mail_thunder_keys,
}



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction
from je_editor import language_wrapper

from test_pioneer.test_pioneer_editor_ui.menu.install_menu.install_utils import install_package

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor


def build_tool_install_menu(ui_we_want_to_set: AutomationEditor):
    ui_we_want_to_set.install_tools_menu = ui_we_want_to_set.install_menu.addMenu(
        language_wrapper.language_word_dict.get("install_menu_tools_install_menu_label"))
    # Try to install Build Tools
    ui_we_want_to_set.install_tool_action = QAction(
        language_wrapper.language_word_dict.get("install_menu_tools_install_build_tools"))
    ui_we_want_to_set.install_tool_action.triggered.connect(
        lambda: install_build_tools(ui_we_want_to_set)
    )
    ui_we_want_to_set.install_tools_menu.addAction(ui_we_want_to_set.install_tool_action)


def install_build_tools(ui_we_want_to_set: AutomationEditor) -> None:
    install_package("setuptools", ui_we_want_to_set)
    install_package("build", ui_we_want_to_set)
    install_package("wheel", ui_we_want_to_set)



import sys
from typing import List, Dict, Type

from PySide6.QtCore import QTimer, QCoreApplication
from PySide6.QtWidgets import QApplication, QWidget
from je_editor import EditorMain, language_wrapper
from qt_material import apply_stylesheet

from test_pioneer.test_pioneer_editor_ui.extend_multi_language.update_language_dict import update_language_dict
from test_pioneer.test_pioneer_editor_ui.menu.build_menubar import add_menu_to_menubar
from test_pioneer.test_pioneer_editor_ui.syntax.syntax_extend import \
    syntax_extend_package

EDITOR_EXTEND_TAB: Dict[str, Type[QWidget]] = {}


class AutomationEditor(EditorMain):

    def __init__(self, debug_mode: bool = False, show_system_tray_ray: bool = False):
        super().__init__(debug_mode, show_system_tray_ray)
        self.current_run_code_window: List[QWidget] = list()
        # Project compiler if user not choose this will use which to find
        self.python_compiler = None
        # Delete JEditor help
        if self.help_menu:
            self.help_menu.deleteLater()
        # System tray change
        if self.show_system_tray_ray:
            self.system_tray.main_window = self
            self.system_tray.setToolTip(language_wrapper.language_word_dict.get("automation_editor_application_name"))
        # Update language_dict
        update_language_dict()
        # Menu
        add_menu_to_menubar(self)
        syntax_extend_package(self)
        # Tab
        for widget_name, widget in EDITOR_EXTEND_TAB.items():
            self.tab_widget.addTab(widget(), widget_name)
        # Title
        self.setWindowTitle(language_wrapper.language_word_dict.get("automation_editor_application_name"))
        if debug_mode:
            close_timer = QTimer(self)
            close_timer.setInterval(10000)
            close_timer.timeout.connect(self.debug_close)
            close_timer.start()

    def closeEvent(self, event) -> None:
        for widget in self.current_run_code_window:
            widget.close()
        super().closeEvent(event)

    @classmethod
    def debug_close(cls) -> None:
        """
        Use to run CI test.
        :return: None
        """
        sys.exit(0)


def start_editor(debug_mode: bool = False, **kwargs) -> None:
    """
    Start editor instance
    :return: None
    """
    new_editor = QCoreApplication.instance()
    if new_editor is None:
        new_editor = QApplication(sys.argv)
    window = AutomationEditor(debug_mode=debug_mode, **kwargs)
    apply_stylesheet(new_editor, theme="dark_amber.xml")
    window.showMaximized()
    try:
        window.startup_setting()
    except Exception as error:
        print(repr(error))
    sys.exit(new_editor.exec())



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor import EditorWidget

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor
from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QTextCharFormat, QColor

from test_pioneer.test_pioneer_editor_ui.syntax.syntax_keyword import \
    package_keyword_list
from test_pioneer.utils.manager.package_manager.package_manager_class import package_manager

from je_editor import syntax_word_setting_dict


def syntax_extend_package(main_window: AutomationEditor) -> None:
    widget = main_window.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        for package in package_manager.syntax_check_list:
            text_char_format = QTextCharFormat()
            text_char_format.setForeground(QColor(255, 255, 0))
            for word in package_keyword_list.get(package):
                # Highlight
                pattern = QRegularExpression(rf"\b{word}\b")
                widget.code_edit.highlighter.highlight_rules.append((pattern, text_char_format))
    for package in package_manager.syntax_check_list:
        syntax_word_setting_dict.update(
            {
                package: {
                    "words": set(package_keyword_list.get(package)),
                    "color": QColor(255, 255, 0)
                }
            }
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction
from je_editor import language_wrapper

from test_pioneer.test_pioneer_editor_ui.menu.menu_utils import open_web_browser

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor


def set_bing_gpt_menu(ui_we_want_to_set: AutomationEditor):
    ui_we_want_to_set.bing_gpt_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("tools_menu_re_edge_gpt_label"))
    ui_we_want_to_set.bing_gpt_menu.help_menu = ui_we_want_to_set.bing_gpt_menu.addMenu(
        language_wrapper.language_word_dict.get("help_label"))
    # Open Doc
    ui_we_want_to_set.bing_gpt_menu.help_menu.open_bing_gpt_menu_doc_action = QAction(
        language_wrapper.language_word_dict.get("tools_menu_re_edge_gpt_doc_label"))
    ui_we_want_to_set.bing_gpt_menu.help_menu.open_bing_gpt_menu_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://reedgegpt.readthedocs.io/en/latest/",
            language_wrapper.language_word_dict.get("tools_menu_re_edge_gpt_doc_tab_label")
        )
    )
    ui_we_want_to_set.bing_gpt_menu.help_menu.addAction(
        ui_we_want_to_set.bing_gpt_menu.help_menu.open_bing_gpt_menu_doc_action
    )
    # Open Github
    ui_we_want_to_set.bing_gpt_menu.help_menu.open_re_edge_gpt_github_action = QAction(
        language_wrapper.language_word_dict.get("tools_menu_re_edge_gpt_github_label"))
    ui_we_want_to_set.bing_gpt_menu.help_menu.open_re_edge_gpt_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Integration-Automation/ReEdgeGPT",
            language_wrapper.language_word_dict.get("tools_menu_re_edge_gpt_github_tab_label")
        )
    )
    ui_we_want_to_set.bing_gpt_menu.help_menu.addAction(
        ui_we_want_to_set.bing_gpt_menu.help_menu.open_re_edge_gpt_github_action
    )



import sys
from os import getcwd
from os import walk
from os.path import abspath
from os.path import join

from PySide6.QtWidgets import QFileDialog, QMainWindow


def get_dir_files_as_list(dir_path: str = getcwd(), default_search_file_extension: str = ".json") -> list:
    """
    get dir file when end with default_search_file_extension
    :param dir_path: which dir we want to walk and get file list
    :param default_search_file_extension: which extension we want to search
    :return: [] if nothing searched or [file1, file2.... files] file was searched
    """
    return [
        abspath(join(dir_path, file)) for root, dirs, files in walk(dir_path)
        for file in files
        if file.endswith(default_search_file_extension.lower())
    ]


def ask_and_get_dir_files_as_list(main_window: QMainWindow, default_search_file_extension: str = ".json") -> list:
    choose_dir = QFileDialog(parent=main_window).getExistingDirectory()
    if choose_dir is not None and choose_dir != "":
        return get_dir_files_as_list(choose_dir, default_search_file_extension)
    else:
        print("Not select any dir", file=sys.stderr)



import json
import queue
import subprocess
import sys
import threading
import typing
from pathlib import Path
from queue import Queue
from threading import Thread

from PySide6.QtCore import QTimer
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.utils.venv_check.check_venv import check_and_choose_venv

from test_pioneer.test_pioneer_editor_ui.show_code_window.code_window import CodeWindow


class TaskProcessManager(object):
    def __init__(
            self,
            main_window: CodeWindow,
            task_done_trigger_function: typing.Callable = None,
            error_trigger_function: typing.Callable = None,
            program_buffer_size: int = 1024,
            program_encoding: str = "utf-8"
    ):
        super().__init__()
        self.compiler_path = None
        # ite_instance param
        self.read_program_error_output_from_thread: [threading.Thread, None] = None
        self.read_program_output_from_thread: [threading.Thread, None] = None
        self.main_window: CodeWindow = main_window
        self.timer: QTimer = QTimer(self.main_window)
        self.still_run_program: bool = True
        self.program_encoding: str = program_encoding
        self.run_output_queue: Queue = Queue()
        self.run_error_queue: Queue = Queue()
        self.process: [subprocess.Popen, None] = None

        self.task_done_trigger_function: typing.Callable = task_done_trigger_function
        self.error_trigger_function: typing.Callable = error_trigger_function
        self.program_buffer_size = program_buffer_size

    def renew_path(self) -> None:
        if self.main_window.python_compiler is None:
            # Renew compiler path
            if sys.platform in ["win32", "cygwin", "msys"]:
                venv_path = Path(str(Path.cwd()) + "/venv/Scripts")
            else:
                venv_path = Path(str(Path.cwd()) + "/venv/bin")
            self.compiler_path = check_and_choose_venv(venv_path)
        else:
            self.compiler_path = self.main_window.python_compiler

    def start_test_process(self, package: str, exec_str: str):
        self.renew_path()
        if sys.platform in ["win32", "cygwin", "msys"]:
            exec_str = json.dumps(exec_str)
            args = [
                self.compiler_path,
                "-m",
                package,
                "--execute_str",
                exec_str
            ]
        else:
            args = " ".join([f"{self.compiler_path}", f"-m {package}", "--execute_str", f"{exec_str}"])
        self.process: subprocess.Popen = subprocess.Popen(
            args,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
            encoding=self.program_encoding
        )
        self.still_run_program = True
        # program output message queue thread
        self.read_program_output_from_thread = Thread(
            target=self.read_program_output_from_process,
            daemon=True
        )
        self.read_program_output_from_thread.start()
        # program error message queue thread
        self.read_program_error_output_from_thread = Thread(
            target=self.read_program_error_output_from_process,
            daemon=True
        )
        self.read_program_error_output_from_thread.start()
        # start Pyside update
        # start timer
        self.main_window.setWindowTitle(package)
        self.main_window.show()
        self.timer = QTimer()
        self.timer.setInterval(100)
        self.timer.timeout.connect(self.pull_text)
        self.timer.start()

    # Pyside UI update method
    def pull_text(self):
        try:
            self.main_window.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    self.main_window.code_result.append(output_message)
            self.main_window.code_result.setTextColor(actually_color_dict.get("error_output_color"))
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    self.main_window.code_result.append(error_message)
            self.main_window.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
        except queue.Empty:
            pass
        if self.process is not None:
            if self.process.returncode == 0:
                if self.timer.isActive():
                    self.timer.stop()
                self.exit_program()
            elif self.process.returncode is not None:
                if self.timer.isActive():
                    self.timer.stop()
                self.exit_program()
            if self.still_run_program:
                # poll return code
                self.process.poll()
        else:
            if self.timer.isActive():
                self.timer.stop()

    # exit program change run flag to false and clean read thread and queue and process
    def exit_program(self):
        self.still_run_program = False
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()
        if self.process is not None:
            self.process.terminate()
            self.main_window.code_result.append(f"Task exit with code {self.process.returncode}")
            self.process = None

    def print_and_clear_queue(self):
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self):
        while self.still_run_program:
            self.process: subprocess.Popen
            program_output_data = self.process.stdout.read(self.program_buffer_size)
            if self.process:
                self.process.stdout.flush()
            if program_output_data.strip() != "":
                self.run_output_queue.put(program_output_data)

    def read_program_error_output_from_process(self):
        while self.still_run_program:
            program_error_output_data = self.process.stderr.read(self.program_buffer_size)
            if self.process:
                self.process.stderr.flush()
            if program_error_output_data.strip() != "":
                self.run_error_queue.put(program_error_output_data)



from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor
from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import EDITOR_EXTEND_TAB
from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import start_editor

__all__ = [
    "start_editor", "AutomationEditor", "EDITOR_EXTEND_TAB"
]



from je_editor import traditional_chinese_word_dict


def update_traditional_chinese_word_dict():
    traditional_chinese_word_dict.update(
        {
            # application name
            "automation_editor_application_name": "Automation Editor",
            # Menubar
            "automation_menu_label": "自動化",
            "install_menu_label": "安裝",
            # Normal label
            "run_label": "運行",
            "help_label": "幫助",
            "project_label": "專案",
            # APITestka Menu
            "apitestka_menu_label": "APITestka",
            "apitestka_run_script_label": "運行 APITestka 腳本",
            "apitestka_run_script_with_send_label": "運行 APITestka 腳本並寄信",
            "apitestka_run_multi_script_label": "運行多個 APITestka 腳本",
            "apitestka_run_multi_script_with_send_label": "運行多個 APITestka 腳本並寄信",
            "apitestka_doc_label": "開啟 APITestka 文件",
            "apitestka_doc_tab_label": "APITestka 文件",
            "apitestka_github_label": "開啟 APITestka GitHub",
            "apitestka_github_tab_label": "APITestka GitHub",
            "apitestka_create_project_label": "建立 APITestka 專案",
            # Autocontrol Menu
            "autocontrol_menu_label": "Autocontrol",
            "autocontrol_run_script_label": "運行 Autocontrol 腳本",
            "autocontrol_run_script_with_send_label": "運行 Autocontrol 腳本並寄信",
            "autocontrol_run_multi_script_label": "運行 Multi Autocontrol 腳本",
            "autocontrol_run_multi_script_with_send_label": "運行 Multi Autocontrol 腳本 腳本並寄信",
            "autocontrol_doc_label": "開啟 Autocontrol 文件",
            "autocontrol_doc_tab_label": "Autocontrol 文件",
            "autocontrol_github_label": "開啟 Autocontrol GitHub",
            "autocontrol_github_tab_label": "Autocontrol GitHub",
            "autocontrol_create_project_label": "建立 Autocontrol 專案",
            "autocontrol_record_menu_label": "紀錄",
            "autocontrol_record_start_label": "紀錄 Start",
            "autocontrol_record_stop_label": "紀錄 Stop",
            # File Automation Menu
            "file_automation_menu_label": "FileAutomation",
            "file_automation_run_script_label": "運行 FileAutomation 腳本",
            "file_automation_run_script_with_send_label": "運行 FileAutomation 腳本並寄信",
            "file_automation_run_multi_script_label": "運行 Multi FileAutomation 腳本",
            "file_automation_run_multi_script_with_send_label": "運行 Multi FileAutomation 腳本 腳本並寄信",
            "file_automation_doc_label": "開啟 FileAutomation 文件",
            "file_automation_doc_tab_label": "FileAutomation 文件",
            "file_automation_github_label": "開啟 FileAutomation GitHub",
            "file_automation_github_tab_label": "FileAutomation GitHub",
            "file_automation_create_project_label": "建立 FileAutomation 專案",
            # Load Density Menu
            "load_density_menu_label": "LoadDensity",
            "load_density_run_script_label": "運行 LoadDensity 腳本",
            "load_density_run_script_with_send_label": "運行 LoadDensity 腳本並寄信",
            "load_density_run_multi_script_label": "運行 Multi LoadDensity 腳本",
            "load_density_run_multi_script_with_send_label": "運行 Multi LoadDensity 腳本 腳本並寄信",
            "load_density_doc_label": "開啟 LoadDensity 文件",
            "load_density_doc_tab_label": "LoadDensity 文件",
            "load_density_github_label": "開啟 LoadDensity GitHub",
            "load_density_github_tab_label": "LoadDensity GitHub",
            "load_density_create_project_label": "建立 LoadDensity 專案",
            # Mail Thunder Menu
            "mail_thunder_menu_label": "MailThunder",
            "mail_thunder_run_script_label": "運行 MailThunder 腳本",
            "mail_thunder_doc_label": "開啟 MailThunder 文件",
            "mail_thunder_doc_tab_label": "MailThunder 文件",
            "mail_thunder_github_label": "開啟 MailThunder GitHub",
            "mail_thunder_github_tab_label": "MailThunder GitHub",
            "mail_thunder_create_project_label": "建立 MailThunder 專案",
            # Webrunner Menu
            "web_runner_menu_label": "WebRunner",
            "web_runner_run_script_label": "運行 WebRunner 腳本",
            "web_runner_run_script_with_send_label": "運行 WebRunner 腳本並寄信",
            "web_runner_run_multi_script_label": "運行 Multi WebRunner 腳本",
            "web_runner_run_multi_script_with_send_label": "運行 Multi WebRunner 腳本 腳本並寄信",
            "web_runner_doc_label": "開啟 WebRunner 文件",
            "web_runner_doc_tab_label": "WebRunner 文件",
            "web_runner_github_label": "開啟 WebRunner GitHub",
            "web_runner_github_tab_label": "WebRunner GitHub",
            "web_runner_create_project_label": "建立 WebRunner 專案",
            # Install Menu
            "install_menu_autocontrol": "安裝 AutoControl",
            "install_menu_apitestka": "安裝 APITestka",
            "install_menu_loaddensity": "安裝 LoadDensity",
            "install_menu_webrunner": "安裝 WebRunner",
            "install_menu_automation_file": "安裝 Automation File",
            "install_menu_mail_thunder": "安裝 MailThunder",
            "install_menu_tools_install_menu_label": "工具",
            "install_menu_tools_install_build_tools": "安裝 Build Tools",
            # Tools Menu
            "tools_menu_re_edge_gpt_label": "ReEdgeGPT",
            "tools_menu_re_edge_gpt_doc_label": "開啟 ReEdgeGPT 文件",
            "tools_menu_re_edge_gpt_doc_tab_label": "ReEdgeGPT 文件",
            "tools_menu_re_edge_gpt_github_label": "開啟 ReEdgeGPT GitHub",
            "tools_menu_re_edge_gpt_github_tab_label": "ReEdgeGPT GitHub",
        }
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor import language_wrapper

from test_pioneer.test_pioneer_editor_ui.menu.menu_utils import open_web_browser

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor
import sys

from PySide6.QtGui import QAction

from test_pioneer.extend.process_executor.mail_thunder.mail_thunder_process import call_mail_thunder


def set_mail_thunder_menu(ui_we_want_to_set: AutomationEditor):
    """
    Build menu include LoadDensity feature.
    :param ui_we_want_to_set: main window to add menu.
    :return: None
    """
    ui_we_want_to_set.mail_thunder_menu = ui_we_want_to_set.automation_menu.addMenu(
        language_wrapper.language_word_dict.get("mail_thunder_menu_label"))
    ui_we_want_to_set.mail_thunder_run_menu = ui_we_want_to_set.mail_thunder_menu.addMenu(
        language_wrapper.language_word_dict.get("run_label"))
    # Run MailThunder
    ui_we_want_to_set.run_mail_thunder_action = QAction(
        language_wrapper.language_word_dict.get("mail_thunder_run_script_label"))
    ui_we_want_to_set.run_mail_thunder_action.triggered.connect(
        lambda: call_mail_thunder(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.mail_thunder_run_menu.addAction(
        ui_we_want_to_set.run_mail_thunder_action
    )
    # Help menu
    ui_we_want_to_set.mail_thunder_help_menu = ui_we_want_to_set.mail_thunder_menu.addMenu(
        language_wrapper.language_word_dict.get("help_label"))
    # Open Doc
    ui_we_want_to_set.open_mail_thunder_doc_action = QAction(
        language_wrapper.language_word_dict.get("mail_thunder_doc_label"))
    ui_we_want_to_set.open_mail_thunder_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://mailthunder.readthedocs.io/en/latest/",
            language_wrapper.language_word_dict.get("mail_thunder_doc_tab_label")
        )
    )
    ui_we_want_to_set.mail_thunder_help_menu.addAction(
        ui_we_want_to_set.open_mail_thunder_doc_action
    )
    # Open Github
    ui_we_want_to_set.open_mail_thunder_github_action = QAction(
        language_wrapper.language_word_dict.get("mail_thunder_github_label"))
    ui_we_want_to_set.open_mail_thunder_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Integration-Automation/MailThunder",
            language_wrapper.language_word_dict.get("mail_thunder_github_tab_label")
        )
    )
    ui_we_want_to_set.mail_thunder_help_menu.addAction(
        ui_we_want_to_set.open_mail_thunder_github_action
    )
    ui_we_want_to_set.mail_thunder_project_menu = ui_we_want_to_set.mail_thunder_menu.addMenu(
        language_wrapper.language_word_dict.get("project_label"))
    # Create Project
    ui_we_want_to_set.create_mail_thunder_project_action = QAction(
        language_wrapper.language_word_dict.get("mail_thunder_create_project_label"))
    ui_we_want_to_set.create_mail_thunder_project_action.triggered.connect(
        create_project
    )
    ui_we_want_to_set.mail_thunder_project_menu.addAction(
        ui_we_want_to_set.create_mail_thunder_project_action
    )


def create_project() -> None:
    try:
        import je_mail_thunder
        package = je_mail_thunder
        if package is not None:
            package.create_project_dir()
    except ImportError as error:
        print(repr(error), file=sys.stderr)



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor import EditorWidget, language_wrapper

from test_pioneer.test_pioneer_editor_ui.menu.menu_utils import open_web_browser

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor
import sys

import je_auto_control
from PySide6.QtGui import QAction

from test_pioneer.extend.process_executor.auto_control.auto_control_process import \
    call_auto_control, call_auto_control_with_send, call_auto_control_multi_file, \
    call_auto_control_multi_file_and_send


def set_autocontrol_menu(ui_we_want_to_set: AutomationEditor):
    """
    Build menu include AutoControl feature.
    :param ui_we_want_to_set: main window to add menu.
    :return: None
    """
    ui_we_want_to_set.autocontrol_menu = ui_we_want_to_set.automation_menu.addMenu(
         language_wrapper.language_word_dict.get("autocontrol_menu_label"))
    ui_we_want_to_set.autocontrol_run_menu = ui_we_want_to_set.autocontrol_menu.addMenu(
        language_wrapper.language_word_dict.get("run_label"))
    # Run AutoControl Script
    ui_we_want_to_set.run_autocontrol_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_run_script_label"))
    ui_we_want_to_set.run_autocontrol_action.triggered.connect(
        lambda: call_auto_control(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.autocontrol_run_menu.addAction(ui_we_want_to_set.run_autocontrol_action)
    # Run AutoControl Script With Send
    ui_we_want_to_set.run_autocontrol_action_with_send = QAction(
        language_wrapper.language_word_dict.get("autocontrol_run_script_with_send_label"))
    ui_we_want_to_set.run_autocontrol_action_with_send.triggered.connect(
        lambda: call_auto_control_with_send(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.autocontrol_run_menu.addAction(
        ui_we_want_to_set.run_autocontrol_action_with_send
    )
    # Run Multi AutoControl Script
    ui_we_want_to_set.run_multi_autocontrol_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_run_multi_script_label"))
    ui_we_want_to_set.run_multi_autocontrol_action.triggered.connect(
        lambda: call_auto_control_multi_file(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.autocontrol_run_menu.addAction(
        ui_we_want_to_set.run_multi_autocontrol_action
    )
    # Run Multi AutoControl Script With Send
    ui_we_want_to_set.run_multi_autocontrol_action_with_send = QAction(
        language_wrapper.language_word_dict.get("autocontrol_run_multi_script_with_send_label")
    )
    ui_we_want_to_set.run_multi_autocontrol_action_with_send.triggered.connect(
        lambda: call_auto_control_multi_file_and_send(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.autocontrol_run_menu.addAction(
        ui_we_want_to_set.run_multi_autocontrol_action_with_send
    )
    ui_we_want_to_set.autocontrol_help_menu = ui_we_want_to_set.autocontrol_menu.addMenu(
        language_wrapper.language_word_dict.get("help_label"))
    # Open Doc
    ui_we_want_to_set.open_autocontrol_doc_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_doc_label"))
    ui_we_want_to_set.open_autocontrol_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://autocontrol.readthedocs.io/en/latest/",
            language_wrapper.language_word_dict.get("autocontrol_doc_tab_label")
        )
    )
    ui_we_want_to_set.autocontrol_help_menu.addAction(
        ui_we_want_to_set.open_autocontrol_doc_action
    )
    # Open Github
    ui_we_want_to_set.open_autocontrol_github_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_github_label"))
    ui_we_want_to_set.open_autocontrol_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Intergration-Automation-Testing/AutoControl",
            language_wrapper.language_word_dict.get("autocontrol_github_tab_label")
        )
    )
    ui_we_want_to_set.autocontrol_help_menu.addAction(
        ui_we_want_to_set.open_autocontrol_github_action
    )
    ui_we_want_to_set.autocontrol_project_menu = ui_we_want_to_set.autocontrol_menu.addMenu(
        language_wrapper.language_word_dict.get("project_label"))
    # Create Project
    ui_we_want_to_set.create_autocontrol_project_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_create_project_label"))
    ui_we_want_to_set.create_autocontrol_project_action.triggered.connect(
        create_project
    )
    ui_we_want_to_set.autocontrol_project_menu.addAction(
        ui_we_want_to_set.create_autocontrol_project_action
    )
    # Record
    ui_we_want_to_set.autocontrol_record_menu = ui_we_want_to_set.autocontrol_menu.addMenu(
        language_wrapper.language_word_dict.get("autocontrol_record_menu_label"))
    ui_we_want_to_set.record_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_record_start_label"))
    ui_we_want_to_set.record_action.triggered.connect(
        je_auto_control.record
    )
    ui_we_want_to_set.autocontrol_record_menu.addAction(
        ui_we_want_to_set.record_action
    )
    # Stop Record
    ui_we_want_to_set.stop_record_action = QAction(
        language_wrapper.language_word_dict.get("autocontrol_record_stop_label"))
    ui_we_want_to_set.stop_record_action.triggered.connect(
        lambda: stop_record(ui_we_want_to_set)
    )
    ui_we_want_to_set.autocontrol_record_menu.addAction(
        ui_we_want_to_set.stop_record_action
    )


def create_project() -> None:
    try:
        package = je_auto_control
        if package is not None:
            package.create_project_dir()
    except ImportError as error:
        print(repr(error), file=sys.stderr)


def stop_record(editor_instance: AutomationEditor):
    widget = editor_instance.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.code_edit.appendPlainText(str(je_auto_control.stop_record()))



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor import BrowserWidget

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor


def open_web_browser(
        automation_editor_instance: AutomationEditor, url: str, tab_name: str) -> None:
    automation_editor_instance.tab_widget.addTab(
        BrowserWidget(start_url=url),
        f"{tab_name}{automation_editor_instance.tab_widget.count()}"
    )



# add command exception
add_command_type_exception_tag: str = "command execute_return_value type should be as method or function"
add_command_not_allow_package_exception_tag: str = "choose to add command package not allow"
# send html report exception
send_html_exception_tag: str = """
make sure you have installed je_mail_thunder \n
can"t send html report check login user and password is correct \n
and current working folder have default_name.html (html report default execute_detail) \n
or you should get the function file_path to read
"""
# test executor exception
auto_control_process_executor_exception_tag: str = "can't run AutoControl"
api_testka_process_executor_exception_tag: str = "can't run APITestka"
web_runner_process_executor_exception_tag: str = "can't run WebRunner"
load_density_process_executor_exception_tag: str = "can't run LoadDensity"
# Install
not_install_exception: str = "Please install package first, Can't find package"
# ui exception
wrong_test_data_format_exception_tag: str = "get the wrong test data format"
# exec exception
exec_error: str = "AutomationEditor exec error"
file_not_fond_error: str = "File not found"
compiler_not_found_error: str = "Compiler not found"
not_install_package_error: str = "not install required package"
# json exception
cant_reformat_json_error: str = "Can't reformat json is type right?"
wrong_json_data_error: str = "Can't parser json"
# XML
cant_read_xml_error: str = "can't read xml"
xml_type_error: str = "xml type error"



from je_editor import english_word_dict


def update_english_word_dict():
    english_word_dict.update(
        {
            # application name
            "automation_editor_application_name": "Automation Editor",
            # Menubar
            "automation_menu_label": "Automation",
            "install_menu_label": "Install",
            # Normal label
            "run_label": "Run",
            "help_label": "HELP",
            "project_label": "Project",
            # APITestka Menu
            "apitestka_menu_label": "APITestka",
            "apitestka_run_script_label": "Run APITestka Script",
            "apitestka_run_script_with_send_label": "Run APITestka With Send",
            "apitestka_run_multi_script_label": "Run Multi APITestka Script",
            "apitestka_run_multi_script_with_send_label": "Run Multi APITestka Script With Send",
            "apitestka_doc_label": "Open APITestka Doc",
            "apitestka_doc_tab_label": "APITestka Doc",
            "apitestka_github_label": "Open APITestka GitHub",
            "apitestka_github_tab_label": "APITestka GitHub",
            "apitestka_create_project_label": "Create APITestka Project",
            # Autocontrol Menu
            "autocontrol_menu_label": "Autocontrol",
            "autocontrol_run_script_label": "Run Autocontrol Script",
            "autocontrol_run_script_with_send_label": "Run Autocontrol With Send",
            "autocontrol_run_multi_script_label": "Run Multi Autocontrol Script",
            "autocontrol_run_multi_script_with_send_label": "Run Multi Autocontrol Script With Send",
            "autocontrol_doc_label": "Open Autocontrol Doc",
            "autocontrol_doc_tab_label": "Autocontrol Doc",
            "autocontrol_github_label": "Open Autocontrol GitHub",
            "autocontrol_github_tab_label": "Autocontrol GitHub",
            "autocontrol_create_project_label": "Create Autocontrol Project",
            "autocontrol_record_menu_label": "Record",
            "autocontrol_record_start_label": "Record Start",
            "autocontrol_record_stop_label": "Record Stop",
            # File Automation Menu
            "file_automation_menu_label": "FileAutomation",
            "file_automation_run_script_label": "Run FileAutomation Script",
            "file_automation_run_script_with_send_label": "Run FileAutomation With Send",
            "file_automation_run_multi_script_label": "Run Multi FileAutomation Script",
            "file_automation_run_multi_script_with_send_label": "Run Multi FileAutomation Script With Send",
            "file_automation_doc_label": "Open FileAutomation Doc",
            "file_automation_doc_tab_label": "FileAutomation Doc",
            "file_automation_github_label": "Open FileAutomation GitHub",
            "file_automation_github_tab_label": "FileAutomation GitHub",
            "file_automation_create_project_label": "Create FileAutomation Project",
            # Load Density Menu
            "load_density_menu_label": "LoadDensity",
            "load_density_run_script_label": "Run LoadDensity Script",
            "load_density_run_script_with_send_label": "Run LoadDensity With Send",
            "load_density_run_multi_script_label": "Run Multi LoadDensity Script",
            "load_density_run_multi_script_with_send_label": "Run Multi LoadDensity Script With Send",
            "load_density_doc_label": "Open LoadDensity Doc",
            "load_density_doc_tab_label": "LoadDensity Doc",
            "load_density_github_label": "Open LoadDensity GitHub",
            "load_density_github_tab_label": "LoadDensity GitHub",
            "load_density_create_project_label": "Create LoadDensity Project",
            # Mail Thunder Menu
            "mail_thunder_menu_label": "MailThunder",
            "mail_thunder_run_script_label": "Run MailThunder Script",
            "mail_thunder_doc_label": "Open MailThunder Doc",
            "mail_thunder_doc_tab_label": "MailThunder Doc",
            "mail_thunder_github_label": "Open MailThunder GitHub",
            "mail_thunder_github_tab_label": "MailThunder GitHub",
            "mail_thunder_create_project_label": "Create MailThunder Project",
            # Webrunner Menu
            "web_runner_menu_label": "WebRunner",
            "web_runner_run_script_label": "Run WebRunner Script",
            "web_runner_run_script_with_send_label": "Run WebRunner With Send",
            "web_runner_run_multi_script_label": "Run Multi WebRunner Script",
            "web_runner_run_multi_script_with_send_label": "Run Multi WebRunner Script With Send",
            "web_runner_doc_label": "Open WebRunner Doc",
            "web_runner_doc_tab_label": "WebRunner Doc",
            "web_runner_github_label": "Open WebRunner GitHub",
            "web_runner_github_tab_label": "WebRunner GitHub",
            "web_runner_create_project_label": "Create WebRunner Project",
            # Install Menu
            "install_menu_autocontrol": "Install AutoControl",
            "install_menu_apitestka": "Install APITestka",
            "install_menu_loaddensity": "Install LoadDensity",
            "install_menu_webrunner": "Install WebRunner",
            "install_menu_automation_file": "Install Automation File",
            "install_menu_mail_thunder": "Install MailThunder",
            "install_menu_tools_install_menu_label": "Tools",
            "install_menu_tools_install_build_tools": "Install Build Tools",
            # Tools Menu
            "tools_menu_re_edge_gpt_label": "ReEdgeGPT",
            "tools_menu_re_edge_gpt_doc_label": "Open ReEdgeGPT Doc",
            "tools_menu_re_edge_gpt_doc_tab_label": "ReEdgeGPT Doc",
            "tools_menu_re_edge_gpt_github_label": "Open ReEdgeGPT GitHub",
            "tools_menu_re_edge_gpt_github_tab_label": "ReEdgeGPT GitHub",
        }
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor import language_wrapper

from test_pioneer.test_pioneer_editor_ui.menu.menu_utils import open_web_browser

if TYPE_CHECKING:
    from test_pioneer.test_pioneer_editor_ui.editor_main.main_ui import AutomationEditor
import sys

from PySide6.QtGui import QAction

from test_pioneer.extend.process_executor.web_runner.web_runner_process import call_web_runner_test, \
    call_web_runner_test_with_send, call_web_runner_test_multi_file, call_web_runner_test_multi_file_and_send


def set_web_runner_menu(ui_we_want_to_set: AutomationEditor):
    """
    Build menu include WebRunner feature.
    :param ui_we_want_to_set: main window to add menu.
    :return: None
    """
    ui_we_want_to_set.web_runner_menu = ui_we_want_to_set.automation_menu.addMenu(
        language_wrapper.language_word_dict.get("web_runner_menu_label"))
    ui_we_want_to_set.web_runner_run_menu = ui_we_want_to_set.web_runner_menu.addMenu(
        language_wrapper.language_word_dict.get("web_runner_menu_label"))
    # Run WEBRunner Script
    ui_we_want_to_set.run_web_runner_action = QAction(
        language_wrapper.language_word_dict.get("web_runner_run_script_label"))
    ui_we_want_to_set.run_web_runner_action.triggered.connect(
        lambda: call_web_runner_test(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.web_runner_run_menu.addAction(ui_we_want_to_set.run_web_runner_action)
    # Run WEBRunner Script With Send
    ui_we_want_to_set.run_web_runner_action_with_send = QAction(
        language_wrapper.language_word_dict.get("web_runner_run_script_with_send_label"))
    ui_we_want_to_set.run_web_runner_action_with_send.triggered.connect(
        lambda: call_web_runner_test_with_send(
            ui_we_want_to_set
        )
    )
    ui_we_want_to_set.web_runner_run_menu.addAction(
        ui_we_want_to_set.run_web_runner_action_with_send
    )
    # Run Multi WEBRunner Script
    ui_we_want_to_set.run_multi_web_runner_action = QAction(
        language_wrapper.language_word_dict.get("web_runner_run_multi_script_label"))
    ui_we_want_to_set.run_multi_web_runner_action.triggered.connect(
        lambda: call_web_runner_test_multi_file(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.web_runner_run_menu.addAction(
        ui_we_want_to_set.run_multi_web_runner_action
    )
    # Run Multi WEBRunner Script With Send
    ui_we_want_to_set.run_multi_web_runner_action_with_send = QAction(
        language_wrapper.language_word_dict.get("web_runner_run_multi_script_with_send_label"))
    ui_we_want_to_set.run_multi_web_runner_action_with_send.triggered.connect(
        lambda: call_web_runner_test_multi_file_and_send(
            ui_we_want_to_set,
        )
    )
    ui_we_want_to_set.web_runner_run_menu.addAction(
        ui_we_want_to_set.run_multi_web_runner_action_with_send
    )
    ui_we_want_to_set.web_runner_help_menu = ui_we_want_to_set.web_runner_menu.addMenu(
        language_wrapper.language_word_dict.get("help_label"))
    # Open Doc
    ui_we_want_to_set.open_web_runner_doc_action = QAction(
        language_wrapper.language_word_dict.get("web_runner_doc_label"))
    ui_we_want_to_set.open_web_runner_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://webrunner.readthedocs.io/en/latest/",
            language_wrapper.language_word_dict.get("web_runner_doc_tab_label")
        )
    )
    ui_we_want_to_set.web_runner_help_menu.addAction(
        ui_we_want_to_set.open_web_runner_doc_action
    )
    # Open Github
    ui_we_want_to_set.open_web_runner_github_action = QAction(
        language_wrapper.language_word_dict.get("web_runner_github_label"))
    ui_we_want_to_set.open_web_runner_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Intergration-Automation-Testing/WebRunner",
            language_wrapper.language_word_dict.get("web_runner_github_tab_label")
        )
    )
    ui_we_want_to_set.web_runner_help_menu.addAction(
        ui_we_want_to_set.open_web_runner_github_action
    )
    ui_we_want_to_set.web_runner_project_menu = ui_we_want_to_set.web_runner_menu.addMenu(
        language_wrapper.language_word_dict.get("project_label"))
    # Create Project
    ui_we_want_to_set.create_web_runner_project_action = QAction(
        language_wrapper.language_word_dict.get("web_runner_create_project_label"))
    ui_we_want_to_set.create_web_runner_project_action.triggered.connect(
        create_project
    )
    ui_we_want_to_set.web_runner_project_menu.addAction(
        ui_we_want_to_set.create_web_runner_project_action
    )


def create_project() -> None:
    try:
        import je_web_runner
        package = je_web_runner
        if package is not None:
            package.create_project_dir()
    except ImportError as error:
        print(repr(error), file=sys.stderr)
